@@ -2,11 +2,14 @@
 
 pub(crate) mod ast;
 mod compiler;
+mod iml;
 mod macros;
 mod parser;
+#[cfg(test)]
+mod test;
 mod usage;
 
 pub use compiler::*;
-pub use macros::*;
-pub use parser::*;
-pub use usage::*;
+use iml::*;
+use parser::*;
+use usage::*;
