@@ -5,9 +5,7 @@ use crate::error::Error;
 use crate::reader::Reader;
 use crate::token::Token;
 use crate::value::Value;
-use crate::vm::*;
-use crate::{ccl, value};
-use crate::{tokay_embed, tokay_embed_item};
+use crate::{ccl, tokay, value};
 
 /**
 Implements a Tokay parser in Tokay itself, using the compiler macros from the macros-module.
@@ -18,7 +16,7 @@ pub struct Parser(Program);
 
 impl Parser {
     pub fn new() -> Self {
-        Self(tokay_embed!({
+        Self(tokay!({
         // ----------------------------------------------------------------------------
 
         // Whitespace & EOL
@@ -36,6 +34,7 @@ impl Parser {
         (T_EOL = {  // end-of-line
             ["\n", _, (Op::Skip)],
             [";", _, (Op::Skip)],
+            ["|", _, (Op::Skip)],
             [(token (Token::EOF)), (Op::Skip)],
             [(peek "}"), (Op::Skip)]
         }),
@@ -59,14 +58,14 @@ impl Parser {
             ["t", (value "\t")],
             ["v", (value "\x0b")],
             [T_OctDigit, T_OctDigit, T_OctDigit,
-                (Rust::new(|context| code_to_char(context, 0, 8)))],
+                (Op::Rust(Rust(|context| code_to_char(context, 0, 8))))],
             ["x", T_HexDigit, T_HexDigit,
-                (Rust::new(|context| code_to_char(context, 1, 16)))],
+                (Op::Rust(Rust(|context| code_to_char(context, 1, 16))))],
             ["u", T_HexDigit, T_HexDigit, T_HexDigit, T_HexDigit,
-                (Rust::new(|context| code_to_char(context, 1, 16)))],
+                (Op::Rust(Rust(|context| code_to_char(context, 1, 16))))],
             ["U", T_HexDigit, T_HexDigit, T_HexDigit, T_HexDigit,
                 T_HexDigit, T_HexDigit, T_HexDigit, T_HexDigit,
-                (Rust::new(|context| code_to_char(context, 1, 16)))],
+                (Op::Rust(Rust(|context| code_to_char(context, 1, 16))))],
             Any
         }),
 
@@ -163,7 +162,7 @@ impl Parser {
         (Capture = {
             ["$", T_Alias, _, (call ast[(value "capture_alias")])],
             ["$", T_Integer, _, (call ast[(value "capture_index")])],
-            ["$", "(", _, Expression, ")", _, (call ast[(value "capture_expr")])],
+            ["$", "(", _, (kle [T_EOL, _]), Expression, ")", _, (call ast[(value "capture_expr")])],
             ["$", (call error[(value "'$': Expecting identifier, integer or (expression)")])]
         }),
 
@@ -194,7 +193,7 @@ impl Parser {
         }),
 
         (CallParameters = {
-            (pos [CallParameter, (opt [",", _])])
+            (pos [CallParameter, (opt [",", _]), (kle [T_EOL, _])])
         }),
 
         // Literals
@@ -219,9 +218,9 @@ impl Parser {
         }),
 
         (Collection = {
-            ["(", _, (pos [Expression, (opt [",", _])]), ")", // no expect ")" here!
+            ["(", _, (kle [T_EOL, _]), (pos [Expression, (opt [",", _]), (kle [T_EOL, _])]), ")", // no expect ")" here!
                 (call ast[(value "sequence")])],
-            ["(", _, (pos [CollectionItem, (opt [",", _])]), (expect ")"),
+            ["(", _, (kle [T_EOL, _]), (pos [CollectionItem, (opt [",", _]), (kle [T_EOL, _])]), (expect ")"),
                 (call ast[(value "sequence")])]
         }),
 
@@ -236,7 +235,7 @@ impl Parser {
 
         (TokenCall = {
             TokenLiteral,
-            [T_Consumable, "(", _, (opt CallParameters), (expect ")"),
+            [T_Consumable, "(", _, (kle [T_EOL, _]), (opt CallParameters), (kle [T_EOL, _]), (expect ")"),
                 (call ast[(value "call")])],
             [T_Consumable, (call ast[(value "call")])],
             Parselet,
@@ -259,16 +258,34 @@ impl Parser {
         // Expression & Flow
 
         (Atomic = {
-            ["(", _, Expression, ")"], // no expect ")" here!
+            ["(", _, (kle [T_EOL, _]), Expression, (kle [T_EOL, _]), ")"], // no expect ")" here!
             Literal,
             Token,
+
+            // if
+            ["if", ___, Expression, (kle [T_EOL, _]), (expect Statement),
+                (opt [(kle [T_EOL, _]), "else", ___, (kle [T_EOL, _]), (expect Statement)]),
+                    (call ast[(value "op_if")])],
+
+            // for
+            //["for", ___, T_Identifier, _, "in", ___, Expression, Statement,
+            //    (call ast[(value "op_for_in")])],
+            ["for", ___, StatementOrEmpty, ";", _, StatementOrEmpty, ";", _, StatementOrEmpty,
+                StatementOrEmpty, (call ast[(value "op_for")])],
+            ["for", ___, (call error[(value "'for': Expecting start; condition; iter; statement")])],
+
+            // loop
+            ["loop", ___, Expression, _, Statement, (call ast[(value "op_loop")])],
+            ["loop", ___, (expect Statement), (call ast[(value "op_loop")])],
+
+            // standard load
             Load
         }),
 
         (Rvalue = {
             [Rvalue, ".", _, T_Alias, (call ast[(value "attribute")])],
             [Rvalue, (pos Subscript), (call ast[(value "rvalue")])],
-            [Rvalue, "(", _, (opt CallParameters), (expect ")"), _,
+            [Rvalue, "(", _, (kle [T_EOL, _]), (opt CallParameters), (expect ")"), _,
                 (call ast[(value "call")])],
             Atomic
         }),
@@ -314,20 +331,6 @@ impl Parser {
         }),
 
         (Expression = {
-            // if
-            ["if", ___, Expression, Statement, (kle [T_EOL, _]), "else", ___, Statement, (call ast[(value "op_ifelse")])],
-            ["if", ___, Expression, Statement, (call ast[(value "op_if")])],
-            ["if", ___, (call error[(value "'if': Expecting condition and statement")])],
-
-            // for
-            ["for", ___, T_Identifier, _, "in", _, Expression, Statement, (call ast[(value "op_for_in")])],
-            ["for", ___, StatementOrVoid, ";", _, StatementOrVoid, ";", _, StatementOrVoid, (opt T_EOL), _,
-                StatementOrVoid, (call ast[(value "op_for")])],
-            ["for", ___, (call error[(value "'for': Expecting start; condition; iter; statement")])],
-
-            // loop
-            ["loop", ___, Expression, _, (opt T_EOL), _, Statement, (call ast[(value "op_loop")])],
-            ["loop", ___, Statement, (call ast[(value "op_loop_forever")])],
 
             // assignment
             [Lvalue, _, "=", (not {">", "="}), //avoid wrongly matching "=>" or "=="
@@ -341,19 +344,22 @@ impl Parser {
             LogicalOr
         }),
 
-        (StatementOrVoid = {
+        (StatementOrEmpty = {
             Statement,
-            (call ast[(value "value_void")])
+            (call ast[(value "op_nop")])
         }),
 
         (Statement = {
             ["accept", ___, (opt Expression), (call ast[(value "op_accept")])],
+            ["break", ___, (opt Expression), (call ast[(value "op_break")])],
+            ["continue", ___, (opt Expression), (call ast[(value "op_continue")])],
+            ["exit", ___, (opt Expression), (call ast[(value "op_exit")])],
             ["next", ___, (call ast[(value "op_next")])],
             ["push", ___, (opt Expression), (call ast[(value "op_push")])],
             ["reject", ___, (call ast[(value "op_reject")])],
             ["repeat", ___, (opt Expression), (call ast[(value "op_repeat")])],
             ["return", ___, (opt Expression), (call ast[(value "op_accept")])],
-            // todo: escape, break, continue?
+            // todo: escape?
 
             [Lvalue, _, "=", (not {">", "="}), //avoid wrongly matching "=>" or "==" here
                 _, (expect Expression), (call ast[(value "assign")])],
@@ -381,8 +387,8 @@ impl Parser {
         }),
 
         (Block = {
-            ["{", _, (pos Instruction), _, (expect "}"), (call ast[(value "block")])],
-            ["{", _, (kle [T_EOL, _]), (expect "}"), (call ast[(value "value_void")])]
+            ["{", _, (kle [T_EOL, _]), "}", (call ast[(value "value_void")])],
+            ["{", _, (pos Instruction), _, (expect "}"), (call ast[(value "block")])]
         }),
 
         // Sequences
@@ -393,22 +399,22 @@ impl Parser {
             Statement
         }),
 
-        (Sequence = {
+        (ImlSequence = {
             [(pos [SequenceItem, (opt [",", _])]), (call ast[(value "sequence")])]
         }),
 
         (SequenceOrExpression = {
             [Expression, (peek T_EOL)],
-            Sequence
+            ImlSequence
         }),
 
         (Instruction = {
-            ["begin", ___, Sequence, (expect T_EOL), (call ast[(value "begin")])],
-            ["end", ___, Sequence, (expect T_EOL), (call ast[(value "end")])],
+            ["begin", ___, ImlSequence, (expect T_EOL), (call ast[(value "begin")])],
+            ["end", ___, ImlSequence, (expect T_EOL), (call ast[(value "end")])],
 
             [T_Identifier, _, ":", _, (expect SequenceOrExpression), (expect T_EOL),
                 (call ast[(value "constant")])],
-            Sequence,
+            ImlSequence,
             [T_EOL, (Op::Skip)]
         }),
 
@@ -422,6 +428,35 @@ impl Parser {
             (expect (token (Token::EOF)), "Parse error, expecting end-of-file"),
             (call ast[(value "main")])]
 
+        // --- Test Environment -----------------------------------------------
+
+        /*
+        (T_EOL = {  // end-of-line
+            [";", (Op::Skip)],
+            [(token (Token::EOF)), (Op::Skip)],
+            [(peek "}"), (Op::Skip)]
+        }),
+
+        (T_Integer = {
+            // todo: implement as built-in Parselet
+            [(token (Token::Chars(ccl!['0' => '9']))), (call ast[(value "value_integer")])]
+        }),
+
+        (Instruction = {
+            T_Integer,
+            [T_EOL, (Op::Skip)]
+        }),
+
+        (Block = {
+            ["{", (pos Instruction), (expect "}"), (call ast[(value "block")])],
+            ["{", (kle T_EOL), (expect "}"), (call ast[(value "value_void")])]
+        }),
+
+        [Block,
+            (expect (token (Token::EOF)), "Parse error, expecting end-of-file"),
+            (call ast[(value "main")])]
+        */
+
         // ----------------------------------------------------------------------------
                     }))
     }
@@ -430,10 +465,14 @@ impl Parser {
         //self.0.dump();
         let mut runtime = Runtime::new(&self.0, &mut reader);
 
+        if let Ok(level) = std::env::var("TOKAY_PARSER_DEBUG") {
+            runtime.debug = level.parse::<u8>().unwrap_or_default();
+        } else {
+            runtime.debug = 0;
+        }
+
         match self.0.run(&mut runtime) {
             Ok(Some(ast)) => {
-                let ast = Value::from_ref(ast).unwrap();
-
                 if ast.get_dict().is_some() {
                     Ok(ast)
                 } else {
