@@ -1,12 +1,97 @@
 //! Unit tests
 
-use crate::compiler::*;
-use crate::token::*;
 use crate::utils::*;
 use crate::value;
 use crate::value::*;
-use crate::vm::*;
-use crate::{tokay_embed, tokay_embed_item};
+
+//use std::env;
+use std::fs::File;
+use std::io::Read;
+use std::process::Command;
+
+// Universal function to run test-cases from files with expected output.
+fn run_testcase(filename: &'static str) {
+    let mut code = String::new();
+
+    match File::open(filename) {
+        // The file is open (no error).
+        Ok(mut file) => {
+            // Read all the file content into a variable (ignoring the result of the operation).
+            file.read_to_string(&mut code).unwrap();
+
+            // The file is automatically closed when is goes out of scope.
+        }
+        // Error handling.
+        Err(error) => {
+            panic!("Error opening file {}: {}", filename, error);
+        }
+    }
+
+    //println!("code = {:?}", code);
+
+    if let Some((_, data)) = code.split_once("#---\n") {
+        let mut params = vec![filename];
+        let mut result = data;
+        let tmp;
+
+        // In case there is another separator, split into input and output
+        if let Some((i, o)) = data.split_once("#---\n") {
+            let input: Vec<&str> = i
+                .split("\n")
+                .filter(|line| line.starts_with("#"))
+                .map(|line| &line[1..])
+                .collect();
+
+            tmp = input.join("\n");
+            params.extend(vec!["--", &tmp]);
+            result = o;
+        }
+
+        //let program = env::args().next().unwrap(); // Doens't work with cargo test
+        let program = "target/debug/tokay";
+
+        let output = Command::new(program)
+            .args(&params)
+            //.env("LS_COLORS", "rs=0:di=38;5;27:mh=44;38;5;15")
+            .output()
+            .expect(&format!(
+                "Failed to run '{} {}'; You have to `cargo run` first!",
+                program, filename
+            ));
+
+        let mut out: Vec<String> = String::from_utf8(output.stdout)
+            .expect("Not UTF-8")
+            .split("\n")
+            .map(|line| line.to_string())
+            .collect();
+        let mut err: Vec<String> = String::from_utf8(output.stderr)
+            .expect("Not UTF-8")
+            .split("\n")
+            .map(|line| line.to_string())
+            .collect();
+
+        println!("out = {:?}", out);
+        println!("err = {:?}", err);
+
+        for line in result.trim().split("\n").into_iter() {
+            assert!(
+                line.starts_with("#"),
+                "Lines in result must start with a #-comment"
+            );
+
+            if line.starts_with("#ERR:") {
+                assert_eq!(err.remove(0), line[5..].to_string());
+            } else {
+                assert_eq!(out.remove(0), line[1..].to_string());
+            }
+        }
+
+        assert!(out.len() == 1, "Some output not consumed: {:?}", out);
+        assert!(err.len() == 1, "Some errors not consumed: {:?}", err);
+    } else {
+        panic!("Testcase invalid, require for a '#---' delimiter.")
+    }
+}
 
 // Tests expression basics ------------------------------------------------------------------------
 
@@ -22,8 +107,7 @@ fn literal() {
             false \
             \"Hello World\" \
             ",
-            "",
-            false
+            ""
         ),
         Ok(Some(value!([1337, 23.5, true, false, "Hello World"])))
     );
@@ -41,9 +125,8 @@ fn expression() {
             7 * 3 \
             21 / 3 \
             1 + 2 * 3 + 4 \
-        ",
-            "",
-            false
+            ",
+            ""
         ),
         Ok(Some(value!([3, 1, 21, 7, 11])))
     );
@@ -57,9 +140,8 @@ fn expression() {
             2.1 * 6.5 \
             13.6 / 15.1 \
             1.1 + 2.2 * 3.3 + 4.4 \
-        ",
-            "",
-            false
+            ",
+            ""
         ),
         Ok(Some(value!([
             0.5,
@@ -80,9 +162,8 @@ fn expression() {
             \"a\" + \"b\" * 3 \
             \"a\" * 9 \
             \"a\" * 3.3 \
-        ",
-            "",
-            false
+            ",
+            ""
         ),
         Ok(Some(value!([
             "ab",
@@ -115,9 +196,8 @@ fn expression() {
             \"a\" != \"a\" \
             \"a\" != \"a\" * 2 \
             \"a\" == \"a\" * 2 \
-        ",
-            "",
-            false
+            ",
+            ""
         ),
         Ok(Some(value!([
             true, true, true, false, false, false, true, false, true, false, true, false, true,
@@ -143,9 +223,8 @@ fn expression() {
             \"42.23\" <= 42.23 \
             \"42.23\" < 42.23 \
             \"42.23\" > 42.23 \
-        ",
-            "",
-            false
+            ",
+            ""
         ),
         Ok(Some(value!([
             true, true, false, false, true, true, false, false, true, false, false, true
@@ -167,9 +246,8 @@ fn expression() {
             b || a && a \
             b || b && a \
             b && b
-        ",
-            "",
-            false
+            ",
+            ""
         ),
         Ok(Some(value!([true, false, true, true, true, false, false])))
     );
@@ -183,8 +261,7 @@ fn expression() {
             !true \
             !!true
             ",
-            "",
-            false
+            ""
         ),
         Ok(Some(value![[(-1), 1, false, true]]))
     );
@@ -206,8 +283,7 @@ fn operations() {
             a /= 2 a \
             a *= 10 a
             ",
-            "",
-            false
+            ""
         ),
         Ok(Some(value![[true, true, 31, 22, 66, 11, 5.5, 55.0]]))
     );
@@ -226,8 +302,7 @@ fn operations() {
             a-- - --a \
             a-- - - --a \
             a",
-            "",
-            false
+            ""
         ),
         Ok(Some(value![[1, 1, 3, 3, 4, 12, 2, 8, 3]]))
     );
@@ -244,8 +319,7 @@ fn variables() {
             a++
             a b
             ",
-            "",
-            false
+            ""
         ),
         Ok(Some(value![[11, 10]]))
     );
@@ -262,8 +336,7 @@ fn variables() {
 
             f
             ",
-            "",
-            false
+            ""
         ),
         Ok(Some(value![[11, 10]]))
     );
@@ -274,8 +347,7 @@ fn variables() {
             "
             10 20 $1 = $2 = 30 ++$1 $2
             ",
-            "",
-            false
+            ""
         ),
         Ok(Some(value![[31, 30, 31, 30]]))
     );
@@ -286,8 +358,7 @@ fn variables() {
             "
             a => 10 b => 20 $a = $b = 30 c => ++$a d => $b
             ",
-            "",
-            false
+            ""
         ),
         Ok(Some(value![["a" => 31, "b" => 30, "c" => 31, "d" => 30]]))
     );
@@ -297,7 +368,7 @@ fn variables() {
 // Test-case for scoping
 fn scoping() {
     assert_eq!(
-        compile_and_run(include_str!("tests/testcase_scopes.tok"), "", false),
+        compile_and_run(include_str!("../tests/test_scopes.tok"), ""),
         Ok(Some(value![[10, 2000, 1072]]))
     );
 }
@@ -315,8 +386,7 @@ fn collections() {
             (1, 2, 3) \
             (42, \"Hello\", 23.5, true, false)
             ",
-            "",
-            false
+            ""
         ),
         Ok(Some(value!([
             [1, 2, 3],
@@ -334,8 +404,7 @@ fn collections() {
             (a => 1, b => 2, c => 3) \
             (a => 42, x * 2 => \"Hello\", c => 23.5)
             ",
-            "",
-            false
+            ""
         ),
         Ok(Some(value!([
             ["a" => 1, "b" => 2, "c" => 3],
@@ -354,25 +423,25 @@ fn token_modifiers() {
 
     // Simple touch
     assert_eq!(
-        compile_and_run("'a' 'b'", s, false),
+        compile_and_run("'a' 'b'", s),
         Ok(Some(value![[["a", "b"], ["a", "b"]]]))
     );
 
     // Touch and match
     assert_eq!(
-        compile_and_run("'a' ''b''", s, false),
+        compile_and_run("'a' ''b''", s),
         Ok(Some(value![["b", "b"]]))
     );
 
     // Match with positive modifier
     assert_eq!(
-        compile_and_run("'a' ''b''+", s, false),
+        compile_and_run("'a' ''b''+", s),
         Ok(Some(value![["b", ["b", "b", "b"]]]))
     );
 
     // Match with kleene and positive modifiers
     assert_eq!(
-        compile_and_run("''a''* ''b''+", s, false),
+        compile_and_run("''a''* ''b''+", s),
         Ok(Some(value![[
             ["a", "b"],
             ["a", ["b", "b", "b"]],
@@ -382,32 +451,56 @@ fn token_modifiers() {
 
     // Touch with kleene and positive modifiers
     assert_eq!(
-        compile_and_run("'a'* ''b''+", s, false),
+        compile_and_run("'a'* ''b''+", s),
         Ok(Some(value![["b", ["b", "b", "b"], ["b", "b"]]]))
     );
 
     // Character classes
     assert_eq!(
-        compile_and_run("[a-z]", &s[..2], false),
+        compile_and_run("[a-z]", &s[..2]),
         Ok(Some(value![["a", "b"]]))
     );
 
     assert_eq!(
-        compile_and_run("[a-z]+", s, false),
+        compile_and_run("[a-z]+", s),
         Ok(Some(value![["ab", "abbb", "bb", "def"]]))
     );
 
     assert_eq!(
-        compile_and_run("[^ ']+", s, false),
+        compile_and_run("[^ ']+", s),
         Ok(Some(value![[
             "ab", "abbb", "bb", "123", "ABC", "456", "def"
         ]]))
     );
 
     // Built-in token
+    assert_eq!(compile_and_run("Integer", s), Ok(Some(value![[123, 456]])));
+
+    // Parsing with sequences and modifiers
+
     assert_eq!(
-        compile_and_run("Integer", s, false),
-        Ok(Some(value![[123, 456]]))
+        compile_and_run("''a'' {''b'' ''c''}* ''d''", "abcbcd"),
+        Ok(Some(value![["a", [["b", "c"], ["b", "c"]], "d"]]))
+    );
+
+    assert_eq!(
+        compile_and_run("''a'' {''b'' ''c''}+ ''d''", "abcbcd"),
+        Ok(Some(value![["a", [["b", "c"], ["b", "c"]], "d"]]))
+    );
+
+    assert_eq!(
+        compile_and_run("''a'' {''b'' ''c''}* ''d''", "ad"),
+        Ok(Some(value![["a", "d"]]))
+    );
+
+    assert_eq!(
+        compile_and_run("''a'' {''b'' ''c''}+ ''d''", "ad"),
+        Ok(None)
+    );
+
+    assert_eq!(
+        compile_and_run("{ Word { ',' _ }? }+", "Hello,   World,  Beta,  Test"),
+        Ok(Some(value![["Hello", "World", "Beta", "Test"]]))
     );
 
     // todo: more token tests, please!
@@ -419,7 +512,7 @@ fn builtin_tokens() {
     let gliders = "Glasflügel Libelle 201b\tG102 Astir  \nVentus_2cT";
 
     assert_eq!(
-        compile_and_run("Identifier", gliders, false),
+        compile_and_run("Identifier", gliders),
         Ok(Some(value!([
             "Glasflügel",
             "Libelle",
@@ -431,17 +524,17 @@ fn builtin_tokens() {
     );
 
     assert_eq!(
-        compile_and_run("Integer", gliders, false),
+        compile_and_run("Integer", gliders),
         Ok(Some(value!([201, 102, 2])))
     );
 
     assert_eq!(
-        compile_and_run("Whitespaces", gliders, false),
+        compile_and_run("Whitespaces", gliders),
         Ok(Some(value!([" ", " ", "\t", " ", "  \n"])))
     );
 
     assert_eq!(
-        compile_and_run("Word", gliders, false),
+        compile_and_run("Word", gliders),
         Ok(Some(value!([
             "Glasflügel",
             "Libelle",
@@ -457,12 +550,12 @@ fn builtin_tokens() {
     let abc = "abc   \tdef  abcabc= ghi abcdef";
 
     assert_eq!(
-        compile_and_run("Word _; ", abc, false),
+        compile_and_run("Word _; ", abc),
         Ok(Some(value![["abc", "def", "abcabc", "ghi", "abcdef"]]))
     );
 
     assert_eq!(
-        compile_and_run("Word __; ", abc, false),
+        compile_and_run("Word __; ", abc),
         Ok(Some(value![["abc", "def", "ghi"]]))
     );
 }
@@ -482,8 +575,7 @@ fn parselet_static_with_args() {
 
             faculty(4)
             ",
-            "",
-            false
+            ""
         ),
         Ok(Some(value!(24)))
     );
@@ -502,8 +594,7 @@ fn parselet_variable_with_args() {
 
             faculty(4)
             ",
-            "",
-            false
+            ""
         ),
         Ok(Some(value!(24)))
     );
@@ -513,7 +604,7 @@ fn parselet_variable_with_args() {
 // Testing left-recursive parselets
 fn parselet_leftrec() {
     assert_eq!(
-        compile_and_run("P: @{ P? ''a'' }\nP", "aaaa", false),
+        compile_and_run("P: @{ P? ''a'' }\nP", "aaaa"),
         Ok(Some(value!([[["a", "a"], "a"], "a"])))
     );
 
@@ -543,18 +634,22 @@ fn parselet_call_error_reporting() {
         let call = format!("f : @a, b=2, c {{ a b c }}\n{}", call);
         println!("calling {:?}, expecting {:?}", call, msg);
 
-        assert_eq!(compile_and_run(&call, "", false), Err(msg.to_owned()));
+        assert_eq!(compile_and_run(&call, ""), Err(msg.to_owned()));
     }
 }
 
+#[test]
+fn kaputt() {
+    assert_eq!(compile_and_run("{{}}", ""), Ok(Some(Value::Void)));
+}
+
 #[test]
 // Testing examples provided in the examples folder
 fn examples() {
     assert_eq!(
         compile_and_run(
             include_str!("../examples/planets.tok"),
-            "Mercury Venus Earth Mars",
-            false
+            "Mercury Venus Earth Mars"
         ),
         Ok(Some(value!([
             "Hello Mercury",
@@ -564,11 +659,11 @@ fn examples() {
         ])))
     );
 
+    /*
     assert_eq!(
         compile_and_run(
             include_str!("../examples/planets2.tok"),
-            "Mercury Venus Earth Mars Jupiter Saturn Uranus Neptune",
-            false
+            "Mercury Venus Earth Mars Jupiter Saturn Uranus Neptune"
         ),
         Ok(Some(value!([
             "Mercury",
@@ -581,33 +676,29 @@ fn examples() {
             "Neptune"
         ])))
     );
+    */
 
     assert_eq!(
-        compile_and_run(include_str!("../examples/expr.tok"), "1+2*3+4", false),
+        compile_and_run(include_str!("../examples/expr.tok"), "1+2*3+4"),
         Ok(Some(value!(11)))
     );
 
     // todo: Would be nice to test against stdout
     assert_eq!(
-        compile_and_run(
-            include_str!("../examples/expr_with_ast.tok"),
-            "1+2*3+4",
-            false
-        ),
+        compile_and_run(include_str!("../examples/expr_with_ast.tok"), "1+2*3+4"),
         Ok(None)
     );
 
     assert_eq!(
         compile_and_run(
             include_str!("../examples/expr_with_spaces.tok"),
-            "1 +  \t 2 \n *  3 + 4",
-            false
+            "1 +  \t 2 \n *  3 + 4"
         ),
         Ok(Some(value!(11)))
     );
 
     assert_eq!(
-        compile_and_run(include_str!("../examples/faculty.tok"), "", false),
+        compile_and_run(include_str!("../examples/faculty.tok"), ""),
         Ok(Some(value!(24)))
     );
 }
@@ -616,12 +707,12 @@ fn examples() {
 // Testing sequence captures
 fn capture() {
     assert_eq!(
-        compile_and_run("'a' 'b' $1 * 2 + $2 * 3", "ab", false),
+        compile_and_run("'a' 'b' $1 * 2 + $2 * 3", "ab"),
         Ok(Some(value!("aabbb")))
     );
 
     assert_eq!(
-        compile_and_run("a=2 'a' 'b' $(a + 1) * 3+ $(a) * 2", "ab", false),
+        compile_and_run("a=2 'a' 'b' $(a + 1) * 3+ $(a) * 2", "ab"),
         Ok(Some(value!("bbbaa")))
     );
 }
@@ -639,8 +730,7 @@ fn begin_end() {
 
             P: @{ 'lol' x = x + 1 x }
             P",
-            "lolalolaalolol",
-            false
+            "lolalolaalolol"
         ),
         Ok(Some(value!([1337, 1, 2, 3, 1338])))
     );
@@ -651,8 +741,7 @@ fn begin_end() {
             begin x = 1
 
             'lol' $1 * x x x = x + 1",
-            "lolAlolBlol",
-            false
+            "lolAlolBlol"
         ),
         Ok(Some(value!([["lol", 1], ["lollol", 2], ["lollollol", 3]])))
     );
@@ -665,8 +754,7 @@ fn begin_end() {
             2 3 4
             end 5
             ",
-            "",
-            false
+            ""
         ),
         Ok(Some(value!([1, [2, 3, 4], 5])))
     )
@@ -676,7 +764,7 @@ fn begin_end() {
 // Testing parselet repeat
 fn repeat() {
     assert_eq!(
-        compile_and_run("P: @{ 'a' repeat $1 }\nP", "aaaa", false),
+        compile_and_run("P: @{ 'a' repeat $1 }\nP", "aaaa"),
         Ok(Some(value!(["a", "a", "a", "a"])))
     );
 
@@ -694,8 +782,7 @@ fn if_else() {
             if false 2 \
             if $1 3 else 4 \
             if !$2 5 else 6",
-            "",
-            false
+            ""
         ),
         Ok(Some(value!([1, 3, 5])))
     );
@@ -711,11 +798,12 @@ fn if_else() {
             if nb 2 \
             if $1 3 else 4 \
             if !$2 5 else 6",
-            "",
-            false
+            ""
         ),
         Ok(Some(value!([1, 3, 5])))
     );
+
+    run_testcase("tests/test_if.tok");
 }
 
 #[test]
@@ -727,8 +815,7 @@ fn push_next() {
             1 2 3 next
             4 5 6 push 7
             ",
-            "",
-            false
+            ""
         ),
         Ok(Some(value!(7)))
     );
@@ -736,31 +823,16 @@ fn push_next() {
     // todo: This test is a stub. Add more tests regarding next and push.
 }
 
-// Tests for compiler behavior --------------------------------------------------------------------
-
-// Universal function to run test-cases with expected errors inside the code.
-fn run_testcase(code: &'static str) {
-    if let Some((code, result)) = code.split_once("#---\n") {
-        let expect = result
-            .trim()
-            .split("\n")
-            .into_iter()
-            .map(|line| {
-                assert!(
-                    line.starts_with("#"),
-                    "Lines in result must start with a comment-#"
-                );
-                line[1..].to_string()
-            })
-            .collect::<Vec<String>>()
-            .join("\n");
-
-        assert_eq!(compile_and_run(code, "", false), Err(expect));
-    } else {
-        panic!("Testcase invalid, require for a '#---' delimiter.")
-    }
+#[test]
+// tests for push and next
+fn loops() {
+    run_testcase("tests/test_loop.tok");
+    run_testcase("tests/test_for.tok");
+    run_testcase("tests/err_break_continue.tok");
 }
 
+// Tests for compiler behavior --------------------------------------------------------------------
+
 #[test]
 // Testing several special parsing constructs and error reporting
 fn compiler_structure() {
@@ -778,16 +850,15 @@ fn compiler_structure() {
             }
 
             a b c
-        ",
-            "",
-            false
+            ",
+            ""
         ),
         Ok(Some(value!(3)))
     );
 
     // Test for invalid input when EOF is expected
     assert_eq!(
-        compile_and_run("{}}", "", false),
+        compile_and_run("{}}", ""),
         Err("Line 1, column 3: Parse error, expecting end-of-file".to_string())
     );
 }
@@ -795,7 +866,7 @@ fn compiler_structure() {
 #[test]
 // Tests for correct identifier names for various value types
 fn compiler_identifier_naming() {
-    run_testcase(include_str!("tests/testcase_compiler_identifier_names.tok"));
+    run_testcase("tests/err_compiler_identifier_names.tok");
 }
 
 #[test]
@@ -804,8 +875,7 @@ fn compiler_unescaping() {
     assert_eq!(
         compile_and_run(
             "\"test\\\\yes\n\\xCA\\xFE\t\\100\\x5F\\u20ac\\U0001F98E\"",
-            "",
-            false
+            ""
         ),
         Ok(Some(value!("test\\yes\nÊþ\t@_€🦎")))
     );
@@ -813,8 +883,7 @@ fn compiler_unescaping() {
     assert_eq!(
         compile_and_run(
             "'hello\\nworld'", // double \ quotation required
-            "hello\nworld",
-            false
+            "hello\nworld"
         ),
         Ok(Some(value!("hello\nworld")))
     );
@@ -822,8 +891,7 @@ fn compiler_unescaping() {
     assert_eq!(
         compile_and_run(
             "[0-9\\u20ac]+", // double \ quotation required
-            "12345€ €12345",
-            false
+            "12345€ €12345"
         ),
         Ok(Some(value!(["12345€", "€12345"])))
     );
@@ -831,8 +899,7 @@ fn compiler_unescaping() {
     assert_eq!(
         compile_and_run(
             "'hello\\nworld'", // double \ quotation required
-            "hello\nworld",
-            false
+            "hello\nworld"
         ),
         Ok(Some(value!("hello\nworld")))
     );
@@ -840,8 +907,7 @@ fn compiler_unescaping() {
     assert_eq!(
         compile_and_run(
             "[0-9\\u20ac]+", // double \ quotation required
-            "12345€ €12345",
-            false
+            "12345€ €12345"
         ),
         Ok(Some(value!(["12345€", "€12345"])))
     );
@@ -854,12 +920,12 @@ fn compiler_unescaping() {
 fn builtins() {
     // ord/chr
     assert_eq!(
-        compile_and_run("i = ord(\"€\"); i chr(i)", "", false),
+        compile_and_run("i = ord(\"€\"); i chr(i)", ""),
         Ok(Some(value![[(8364 as usize), "€"]]))
     );
 
     assert_eq!(
-        compile_and_run("ord(\"12\")", "", false),
+        compile_and_run("ord(\"12\")", ""),
         Err(
             "Line 1, column 1: ord() expected single character, but received string of length 2"
                 .to_string()
@@ -867,7 +933,7 @@ fn builtins() {
     );
 
     assert_eq!(
-        compile_and_run("ord(\"\")", "", false),
+        compile_and_run("ord(\"\")", ""),
         Err(
             "Line 1, column 1: ord() expected single character, but received string of length 0"
                 .to_string()
@@ -890,8 +956,7 @@ fn builtins_str() {
             \"hello world\".replace(\"l\").upper() \
             \"Tokay\".upper()[1] \
             ",
-            "",
-            false
+            ""
         ),
         Ok(Some(value![[
             "ABCÄÖÜ",
@@ -905,86 +970,3 @@ fn builtins_str() {
         ]]))
     );
 }
-
-// Tests for parsing and packrat features ---------------------------------------------------------
-
-/*
-    Below are some tests that provide indirect left-recursion.
-
-    They currently don't work properly due to the following reason:
-    For indirect left-recursion in packrat parsing, one rule in the
-    grammar's graph must be declared as "leading", so that subsequent,
-    even left-recursive parselets are considered as not left-recursive.
-
-
-    An implementation of an solution for this issue can be found in
-    the pegen parser generator from Python:
-
-    https://github.com/python/cpython/blob/main/Tools/peg_generator/pegen/parser_generator.py
-
-    Tokay won't take care of this right now as it is an edge-case
-    and also more complex, as Tokay does not directly implements a
-    grammar.
-*/
-
-#[test]
-fn parser_indirectleftrec() {
-    let program = tokay_embed!({
-        (X = {
-            [Y, (MATCH "c")]
-        }),
-        (Y = {
-            [Z, (MATCH "b")]
-            //Void
-        }),
-        (Z = {
-            X,
-            Y,
-            (MATCH "a")
-        }),
-        Z
-    });
-
-    println!("{:#?}", program.run_from_str("aaabc"));
-}
-
-#[test]
-fn parser_leftrec() {
-    /*
-    let program = tokay_embed!({
-        (X = {
-            [X, (MATCH "b")],
-            (MATCH "a")
-        }),
-
-        X
-    });
-    */
-
-    let program = tokay_embed!({
-        (Y = {
-            X,
-            (MATCH "a")
-        }),
-        (X = {
-            [Y, (MATCH "b")]
-        }),
-        X
-    });
-
-    /*
-    let program = tokay_embed!({
-        (Factor = {
-            ["(", (pos [Expression]), ")"],
-            (token (Token::Chars(ccl!['0'..='9'])))
-        }),
-        (Expression = {
-            [Expression, "+", Expression],
-            Factor
-        }),
-        Expression
-    });
-    */
-
-    println!("{:#?}", program.run_from_str("abb"));
-}
